@@ -27,7 +27,7 @@ const ToolBoxThings = imports.ui.toolBoxThings;
 const PageManager = imports.model.pageManager;
 
 const _LAYER_PAGE    = 0.1;
-const _LAYER_TOOLBOX = 0.2;
+const _LAYER_TOOLBAR = 0.2;
 
 const _SHOW_SPINNER_TIME = 0.5;
 const _HIDE_SPINNER_TIME = 0.5;
@@ -36,7 +36,7 @@ const _SET_CURRENT_PAGE_TIME = 0.4;
 const _SET_NEXT_PAGE_TRANSITION = 'easeInCubic';
 const _SET_PREVIOUS_PAGE_TRANSITION = 'easeOutCubic';
 
-const _SPINNER_SIZE = 30;
+const _SPINNER_SIZE = 20;
 
 function MainWindow(args) {
     this._init(args);
@@ -69,7 +69,8 @@ MainWindow.prototype = {
         this._createContentBox();
 
         this._createContext();
-        this._createToolbar();
+        this._createMainToolbar();
+        this._createSelectionToolbar();
         this._createSpinner();
 
         // show loading screen as soon as possible
@@ -78,6 +79,8 @@ MainWindow.prototype = {
     },
 
     _createGtkWindow : function() {
+        Gtk.Window.set_default_icon_name("the-board");
+
         this._gtkWindow =
             new Gtk.Window({ type: Gtk.WindowType.TOPLEVEL,
                              typeHint: Gdk.WindowTypeHint.DIALOG,
@@ -205,10 +208,10 @@ MainWindow.prototype = {
                                                       this._onStartupBoxTimeout))
     },
 
-    _createToolbar : function() {
+    _createMainToolbar : function() {
         this._mainToolbar = new Toolbar.Toolbar();
 
-        this._mainToolbar.actor.depth = _LAYER_TOOLBOX;
+        this._mainToolbar.actor.depth = _LAYER_TOOLBAR;
 
         this._createToolBoxPages();
         this._createToolBoxThings();
@@ -224,6 +227,25 @@ MainWindow.prototype = {
         this._updateActiveToolbar();
     },
 
+    _createSelectionToolbar : function() {
+        this._selectionToolbar =
+            new Toolbar.Toolbar({ title: Gettext.gettext("Selection"),
+                                  visible: false });
+
+        this._selectionToolbar.actor.depth = _LAYER_TOOLBAR;
+
+        this._createToolBoxAlign();
+        this._createToolBoxDistribute();
+        this._createToolBoxRemove();
+
+        this._contentBox.append(this._selectionToolbar.actor,
+                                Tb.BoxPackFlags.FIXED);
+
+        this._contentBox.set_fixed_child_align(this._selectionToolbar.actor,
+                                               Tb.BoxAlignment.CENTER,
+                                               Tb.BoxAlignment.START);
+    },
+
     _createSpinner : function() {
         this._spinnerBox =
             new Tb.Box({ orientation: Tb.BoxOrientation.VERTICAL,
@@ -254,6 +276,59 @@ MainWindow.prototype = {
         this._mainToolbar.addToolBox(this._toolBoxPages);
     },
 
+    _createToolBoxAlign : function() {
+        this._toolBoxAlign =
+            new ToolBox.ToolBox({ title: Gettext.gettext("Align") });
+
+        this._toolBoxAlign.addButton({ label: Gettext.gettext("Left"),
+                                       actionName: "align",
+                                       actionArgs: { alignment: Page.Alignment.LEFT } });
+
+        this._toolBoxAlign.addButton({ label: Gettext.gettext("Right"),
+                                       actionName: "align",
+                                       actionArgs: { alignment: Page.Alignment.RIGHT } });
+
+        this._toolBoxAlign.addButton({ label: Gettext.gettext("Top"),
+                                       actionName: "align",
+                                       actionArgs: { alignment: Page.Alignment.TOP } });
+
+        this._toolBoxAlign.addButton({ label: Gettext.gettext("Bottom"),
+                                       actionName: "align",
+                                       actionArgs: { alignment: Page.Alignment.BOTTOM } });
+
+        this._selectionToolbar.addToolBox(this._toolBoxAlign);
+    },
+
+    _createToolBoxDistribute : function() {
+        this._toolBoxDistribute =
+            new ToolBox.ToolBox({ title: Gettext.gettext("Distribute") });
+
+        let verticalArgs =
+            { label: Gettext.gettext("Vertical"),
+              actionName: "distribute",
+              actionArgs: { orientation: Page.Orientation.VERTICAL } };
+
+        this._toolBoxDistribute.addButton(verticalArgs);
+
+        let horizontalArgs =
+            { label: Gettext.gettext("Horizontal"),
+              actionName: "distribute",
+              actionArgs: { orientation: Page.Orientation.HORIZONTAL } };
+
+        this._toolBoxDistribute.addButton(horizontalArgs);
+
+        this._selectionToolbar.addToolBox(this._toolBoxDistribute);
+    },
+
+    _createToolBoxRemove : function() {
+        this._toolBoxRemove = new ToolBox.ToolBox();
+
+        this._toolBoxRemove.addButton({ label: Gettext.gettext("Remove"),
+                                        actionName: "remove" });
+
+        this._selectionToolbar.addToolBox(this._toolBoxRemove);
+    },
+
     _createToolBoxThings : function() {
         this._toolBoxThings =
             new ToolBoxThings.ToolBoxThings({ mainWindow: this });
@@ -349,13 +424,13 @@ MainWindow.prototype = {
         if (this._activeToolbar) {
             this._toolbarActiveChangedId =
                 this._activeToolbar.connect("active-changed",
-                                                 Lang.bind(this,
-                                                           this._onToolbarActiveChanged));
+                                            Lang.bind(this,
+                                                      this._onToolbarActiveChanged));
 
             this._toolbarActionId =
                 this._activeToolbar.connect("action",
-                                                 Lang.bind(this,
-                                                           this._onToolbarAction));
+                                            Lang.bind(this,
+                                                      this._onToolbarAction));
         }
     },
 
@@ -365,6 +440,9 @@ MainWindow.prototype = {
         if (this._currentPage &&
             this._currentPage.activeThing) {
             activeToolbar = this._thingToolbar;
+        } else if (this._currentPage &&
+                   this._currentPage.selectedThings.length > 1) {
+            activeToolbar = this._selectionToolbar;
         } else {
             activeToolbar = this._mainToolbar;
         }
@@ -406,7 +484,7 @@ MainWindow.prototype = {
                                      { mainWindow: this });
 
         if (thingToolbar) {
-            thingToolbar.actor.depth = _LAYER_TOOLBOX;
+            thingToolbar.actor.depth = _LAYER_TOOLBAR;
 
             this._toolbarsByThingId[thingId] = thingToolbar;
 
@@ -461,10 +539,17 @@ MainWindow.prototype = {
     _updateWindowSize : function() {
         let screen = Gdk.Screen.get_default();
 
-        let newWidth = Math.round(screen.get_width() * 0.9);
-        let newHeight = Math.round(screen.get_height() * 0.9);
+        let newWidth = Math.round(screen.get_width() * 0.84);
+        if (newWidth % 2) {
+            newWidth -= 1;
+        }
 
-        this._gtkWindow.defaultWidth = Math.min(1115, newWidth);
+        let newHeight = Math.round(screen.get_height() * 0.9);
+        if (newHeight % 2) {
+            newWidth -= 1;
+        }
+
+        this._gtkWindow.defaultWidth = Math.min(1114, newWidth);
         this._gtkWindow.defaultHeight = Math.min(720, newHeight);
     },
 
@@ -652,8 +737,14 @@ MainWindow.prototype = {
     },
 
     _onToolbarActiveChanged : function() {
-        if (this._currentPage && this._activeToolbar.active) {
+        if (!this._currentPage) {
+            return;
+        }
+
+        if (this._activeToolbar.active) {
             this._currentPage.setActiveThing(null);
+        } else {
+            this._mainBox.grab_key_focus();
         }
     },
 
@@ -664,10 +755,11 @@ MainWindow.prototype = {
             return;
         }
 
-        if (this._currentPage &&
-            this._currentPage.activeThing) {
-            this._currentPage.activeThing.doAction(actionName, actionArgs);
+        if (!this._currentPage) {
+            return;
         }
+
+        this._currentPage.doAction(actionName, actionArgs);
     },
 
     _onClipboardTextReceived : function(clipboard, text, data) {
@@ -718,8 +810,14 @@ MainWindow.prototype = {
 
         if (isControl && text == 'v') {
             this._pasteClipboardContent();
+        } else if (isControl && text == 'a') {
+            this._currentPage.selectAllThings();
         } else if (key == Clutter.Escape) {
-            this._hideGtkWindow();
+            if (this._currentPage.selectedThings.length > 0) {
+                this._currentPage.unselectAllThings();
+            } else {
+                this._hideGtkWindow();
+            }
         } else if (text == 't') {
             this._currentPage.addThingFromState({ id: 'note' });
             return true;
@@ -732,6 +830,9 @@ MainWindow.prototype = {
         } else if (text == 'v') {
             this._currentPage.addThingFromState({ id: 'video' });
             return true;
+        } else if (text == 's') {
+            this._currentPage.addThingFromState({ id: 'sound' });
+            return true;
         } else if (text == 'q') {
             this._quitApp();
             return true;
@@ -782,6 +883,10 @@ MainWindow.prototype = {
         }
     },
 
+    _onPageSelectedThingsChanged : function() {
+        this._updateActiveToolbar();
+    },
+
     setCurrentPage : function(pageModel) {
         if (this._currentPage &&
             this._currentPage.model.id == pageModel.id) {
@@ -799,6 +904,9 @@ MainWindow.prototype = {
 
             oldCurrentPage.disconnect(this._pageLoadedChangedId);
             delete this._pageLoadedChangedId;
+
+            oldCurrentPage.disconnect(this._pageSelectedThingsChangedId);
+            delete this._pageSelectedThingsChangedId;
         }
 
         this._currentPage = new Page.Page({ context: this._context,
@@ -820,6 +928,10 @@ MainWindow.prototype = {
             this._currentPage.connect("loaded-changed",
                                       Lang.bind(this, this._onPageLoadedChanged));
 
+        this._pageSelectedThingsChangedId =
+            this._currentPage.connect("selected-things-changed",
+                                      Lang.bind(this, this._onPageSelectedThingsChanged));
+
         if (oldCurrentPage) {
             let onModelSaved = function() {
                 this._setSavingOldCurrentPage(false);
@@ -878,6 +990,11 @@ MainWindow.prototype = {
             delete this._pageLoadedChangedId;
         }
 
+        if (this._pageSelectedThingsChangedId) {
+            this._currentPage.disconnect(this._pageSelectedThingsChangedId);
+            delete this._pageSelectedThingsChangedId;
+        }
+
         if (this._mainBox) {
             this._mainBox.destroy();
             delete this._mainBox;
