@@ -18,6 +18,14 @@ GType tb_box_alignment_get_type (void) G_GNUC_CONST;
 GType tb_box_orientation_get_type (void) G_GNUC_CONST;
 #define TB_TYPE_BOX_ORIENTATION (tb_box_orientation_get_type())
 
+/* enumerations from "tb/tb-sound-player.h" */
+GType tb_sound_player_state_get_type (void) G_GNUC_CONST;
+#define TB_TYPE_SOUND_PLAYER_STATE (tb_sound_player_state_get_type())
+
+/* enumerations from "tb/tb-sound-recorder.h" */
+GType tb_sound_recorder_state_get_type (void) G_GNUC_CONST;
+#define TB_TYPE_SOUND_RECORDER_STATE (tb_sound_recorder_state_get_type())
+
 G_END_DECLS
 
 #endif /* !__TB_ENUM_TYPES_H__ */
